@@ -6,10 +6,10 @@
 from health_azure.utils import (download_checkpoints_from_run_id, download_files_from_run_id,
                                 download_from_datastore, fetch_run, get_most_recent_run, is_running_in_azure_ml,
                                 set_environment_variables_for_multi_node, split_recovery_id, torch_barrier,
-                                upload_to_datastore, RUN_CONTEXT)
+                                upload_to_datastore, RUN_CONTEXT, aggregate_hyperdrive_metrics)
 from health_azure.datasets import DatasetConfig
 from health_azure.himl import (AzureRunInfo, create_run_configuration, create_script_run, get_workspace, submit_run,
-                               submit_to_azure_if_needed)
+                               submit_to_azure_if_needed, create_crossval_hyperdrive_config)
 
 __all__ = [
     "AzureRunInfo",
@@ -30,4 +30,6 @@ __all__ = [
     "submit_to_azure_if_needed",
     "torch_barrier",
     "upload_to_datastore",
+    "create_crossval_hyperdrive_config",
+    "aggregate_hyperdrive_metrics"
 ]
