@@ -10,12 +10,14 @@ import json
 import logging
 import os
 
+import pandas as pd
 import param
 import re
 from argparse import ArgumentParser, OPTIONAL
+from collections import defaultdict
 from itertools import islice
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Optional, Tuple, Type, TypeVar, Union, Set
+from typing import Any, Callable, DefaultDict, Dict, List, Optional, Tuple, Type, TypeVar, Union, Set
 
 import conda_merge
 import ruamel.yaml
@@ -24,6 +26,7 @@ from azureml.core import Environment, Experiment, Run, Workspace, get_run
 from azureml.core.authentication import InteractiveLoginAuthentication, ServicePrincipalAuthentication
 from azureml.core.conda_dependencies import CondaDependencies
 from azureml.data.azure_storage_datastore import AzureBlobDatastore
+from azureml.train.hyperdrive import HyperDriveRun
 
 T = TypeVar('T')
 
@@ -55,6 +58,18 @@ ENV_LOCAL_RANK = "LOCAL_RANK"
 RUN_CONTEXT = Run.get_context()
 WORKSPACE_CONFIG_JSON = "config.json"
 
+# By default, define several environment variables that work around known issues in the software stack
+DEFAULT_ENVIRONMENT_VARIABLES = {
+    "AZUREML_OUTPUT_UPLOAD_TIMEOUT_SEC": "3600",
+    # Occasionally uploading data during the run takes too long, and makes the job fail. Default is 300.
+    "AZUREML_RUN_KILL_SIGNAL_TIMEOUT_SEC": "900",
+    "MKL_SERVICE_FORCE_INTEL": "1",
+    # Switching to a new software stack in AML for mounting datasets
+    "RSLEX_DIRECT_VOLUME_MOUNT": "true",
+    "RSLEX_DIRECT_VOLUME_MOUNT_MAX_CACHE_SIZE": "1",
+    "DATASET_MOUNT_CACHE_SIZE": "1",
+}
+
 PathOrString = Union[Path, str]
 
 
@@ -760,13 +775,7 @@ def create_python_environment(conda_environment_file: Path,
         conda_dependencies.set_pip_option("--extra-index-url https://pypi.org/simple")
     # By default, define several environment variables that work around known issues in the software stack
     environment_variables = {
-        "AZUREML_OUTPUT_UPLOAD_TIMEOUT_SEC": "3600",
-        # Occasionally uploading data during the run takes too long, and makes the job fail. Default is 300.
-        "AZUREML_RUN_KILL_SIGNAL_TIMEOUT_SEC": "900",
-        "MKL_SERVICE_FORCE_INTEL": "1",
-        # Switching to a new software stack in AML for mounting datasets
-        "RSLEX_DIRECT_VOLUME_MOUNT": "true",
-        "RSLEX_DIRECT_VOLUME_MOUNT_MAX_CACHE_SIZE": "1",
+        **DEFAULT_ENVIRONMENT_VARIABLES,
         **(environment_variables or {})
     }
     # See if this package as a whl exists, and if so, register it with AzureML environment.
@@ -1252,3 +1261,61 @@ def torch_barrier() -> None:
         return
     if distributed.is_available() and distributed.is_initialized():
         distributed.barrier()
+
+
+def aggregate_hyperdrive_metrics(run_id: str, child_run_arg_name: str,
+                                 aml_workspace: Optional[Workspace] = None,
+                                 workspace_config_path: Optional[Path] = None) -> pd.DataFrame:
+    """
+    For a given HyperDrive run id, retrieves the metrics from each of its children and then aggregates it.
+    Returns a DataFrame where each column is one child run, and each row is a metric logged by that child run.
+    For example, for a HyperDrive run with 2 children, where each logs epoch, accuracy and loss, the result
+    would look like:
+
+    |              | 0               | 1                  |
+    |--------------|-----------------|--------------------|
+    | epoch        | [1, 2, 3]       | [1, 2, 3]          |
+    | accuracy     | [0.7, 0.8, 0.9] | [0.71, 0.82, 0.91] |
+    | loss         | [0.5, 0.4, 0.3] | [0.45, 0.37, 0.29] |
+
+    here each column is one of the splits/ child runs, and each row is one of the metrics you have logged to the run.
+
+    It is possible to log rows and tables in Azure ML by calling run.log_table and run.log_row respectively.
+    In this case, the DataFrame will contain a Dictionary entry instead of a list, where the keys are the
+    table columns (or keywords provided to log_row), and the values are the table values. e.g.
+
+    |                | 0                                        | 1                                         |
+    |----------------|------------------------------------------|-------------------------------------------|
+    | accuracy_table |{'epoch': [1, 2], 'accuracy': [0.7, 0.8]} | {'epoch': [1, 2], 'accuracy': [0.8, 0.9]} |
+
+    It is also possible to log plots in Azure ML by calling run.log_image and passing in a matplotlib plot. In
+    this case, the DataFrame will contain a string representing the path to the artifact that is generated by AML
+    (the saved plot in the Logs & Outputs pane of your run on the AML portal). E.g.
+
+    |                | 0                                       | 1                                     |
+    |----------------|-----------------------------------------|---------------------------------------|
+    | accuracy_plot  | aml://artifactId/ExperimentRun/dcid.... | aml://artifactId/ExperimentRun/dcid...|
+
+    :param run_id: The run id (type: str) of the parent run. The type of this run must be an AML HyperDriveRun
+    :param child_run_arg_name: the name of the argument given to each child run to denote its position relative
+        to other child runs (e.g. this arg could equal 'child_run_index' - then each of your child runs should expect
+        to receive the arg '--child_run_index' with a value <= the total number of child runs)
+    :param aml_workspace: If provided this is returned as the AzureML Workspace.
+    :param workspace_config_path: If not provided with an AzureML Workspace, then load one given the information in this
+        config
+    :return: A Pandas DataFrame containing the aggregated metrics from each child run
+    """
+    workspace = get_workspace(aml_workspace=aml_workspace, workspace_config_path=workspace_config_path)
+    run = get_aml_run_from_run_id(run_id, aml_workspace=workspace)
+    assert isinstance(run, HyperDriveRun)
+    metrics: DefaultDict = defaultdict()
+    for child_run in run.get_children():
+        child_run_metrics = child_run.get_metrics()
+        child_run_crossval_split = json.loads(child_run.tags.get('hyperparameters')).get(child_run_arg_name)
+        for k, v in child_run_metrics.items():
+            if k not in metrics:
+                metrics[k] = {}
+            metrics[k][child_run_crossval_split] = v
+
+    df = pd.DataFrame.from_dict(metrics, orient="index")
+    return df
